(function(global, factory) {
  typeof exports === "object" && typeof module !== "undefined" ? module.exports = factory(require("vue"), require("konva")) : typeof define === "function" && define.amd ? define(["vue", "konva"], factory) : (global = typeof globalThis !== "undefined" ? globalThis : global || self, global.vueKonva = factory(global.Vue, global.Konva));
})(this, function(vue, Konva) {
  "use strict";
  const _interopDefaultLegacy = (e) => e && typeof e === "object" && "default" in e ? e : { default: e };
  const Konva__default = /* @__PURE__ */ _interopDefaultLegacy(Konva);
  function updatePicture(node) {
    if (!Konva__default.default.autoDrawEnabled) {
      var drawingNode = node.getLayer() || node.getStage();
      drawingNode && drawingNode.batchDraw();
    }
  }
  const propsToSkip = { key: true, style: true, elm: true, isRootInsert: true };
  const EVENTS_NAMESPACE$1 = ".vue-konva-event";
  function applyNodeProps(vueComponent, props = {}, oldProps = {}, useStrict) {
    const instance = vueComponent.__konvaNode;
    var updatedProps = {};
    var hasUpdates = false;
    for (let key in oldProps) {
      if (propsToSkip[key]) {
        continue;
      }
      var isEvent = key.slice(0, 2) === "on";
      var propChanged = oldProps[key] !== props[key];
      if (isEvent && propChanged) {
        var eventName = key.substr(2).toLowerCase();
        if (eventName.substr(0, 7) === "content") {
          eventName = "content" + eventName.substr(7, 1).toUpperCase() + eventName.substr(8);
        }
        instance.off(eventName + EVENTS_NAMESPACE$1, oldProps[key]);
      }
      var toRemove = !props.hasOwnProperty(key);
      if (toRemove) {
        instance.setAttr(key, void 0);
      }
    }
    for (let key in props) {
      if (propsToSkip[key]) {
        continue;
      }
      let isEvent2 = key.slice(0, 2) === "on";
      var toAdd = oldProps[key] !== props[key];
      if (isEvent2 && toAdd) {
        let eventName2 = key.substr(2).toLowerCase();
        if (eventName2.substr(0, 7) === "content") {
          eventName2 = "content" + eventName2.substr(7, 1).toUpperCase() + eventName2.substr(8);
        }
        if (props[key]) {
          instance.off(eventName2 + EVENTS_NAMESPACE$1);
          instance.on(eventName2 + EVENTS_NAMESPACE$1, props[key]);
        }
      }
      if (!isEvent2 && (props[key] !== oldProps[key] || useStrict && props[key] !== instance.getAttr(key))) {
        hasUpdates = true;
        updatedProps[key] = props[key];
      }
    }
    if (hasUpdates) {
      instance.setAttrs(updatedProps);
      updatePicture(instance);
    }
  }
  const componentPrefix = "v";
  function findParentKonva(instance) {
    function re(instance2) {
      if (instance2.__konvaNode) {
        return instance2;
      }
      if (instance2.parent) {
        return re(instance2.parent);
      }
      console.error("vue-konva error: Can not find parent node");
      return {};
    }
    return re(instance.parent);
  }
  function findKonvaNode(instance) {
    var _a;
    if (!(instance == null ? void 0 : instance.component)) {
      return null;
    }
    return ((_a = instance == null ? void 0 : instance.component) == null ? void 0 : _a.__konvaNode) || findKonvaNode(instance.component.subTree);
  }
  function checkTagAndGetNode(instance) {
    const { el, component } = instance;
    const __konvaNode = findKonvaNode(instance);
    if ((el == null ? void 0 : el.tagName) && component && !__konvaNode) {
      const name = el && el.tagName.toLowerCase();
      console.error(
        `vue-konva error: You are trying to render "${name}" inside your component tree. Looks like it is not a Konva node. You can render only Konva components inside the Stage.`
      );
      return null;
    }
    return __konvaNode;
  }
  function getChildren(instance) {
    const collection = [];
    if (instance.children) {
      instance.children.forEach((child) => {
        if (!child.component && Array.isArray(child.children)) {
          collection.push(...child.children);
        }
        if (child.component) {
          collection.push(child);
        }
      });
    }
    return collection;
  }
  function checkOrder(subTree, konvaNode) {
    const children = getChildren(subTree);
    const nodes = [];
    children.forEach((child) => {
      const konvaNode2 = checkTagAndGetNode(child);
      if (konvaNode2) {
        nodes.push(konvaNode2);
      }
    });
    let needRedraw = false;
    nodes.forEach((konvaNode2, index) => {
      if (konvaNode2.getZIndex() !== index) {
        konvaNode2.setZIndex(index);
        needRedraw = true;
      }
    });
    if (needRedraw) {
      updatePicture(konvaNode);
    }
  }
  const Stage = {
    props: {
      config: {
        type: Object,
        default: function() {
          return {};
        }
      },
      __useStrictMode: {
        type: Boolean
      }
    },
    inheritAttrs: false,
    setup(props, { attrs, slots, expose, emits }) {
      const instance = vue.getCurrentInstance();
      const oldProps = vue.reactive({});
      const container = vue.ref(null);
      const __konvaNode = new window.Konva.Stage({
        width: props.config.width,
        height: props.config.height,
        container: document.createElement("div")
      });
      instance.__konvaNode = __konvaNode;
      uploadKonva();
      function getNode() {
        return instance.__konvaNode;
      }
      function getStage() {
        return instance.__konvaNode;
      }
      function uploadKonva() {
        const existingProps = oldProps || {};
        const newProps = {
          ...attrs,
          ...props.config
        };
        applyNodeProps(instance, newProps, existingProps, props.__useStrictMode);
        Object.assign(oldProps, newProps);
      }
      vue.onMounted(() => {
        container.value.innerHTML = "";
        __konvaNode.container(container.value);
        uploadKonva();
      });
      vue.onUpdated(() => {
        uploadKonva();
        checkOrder(instance.subTree, __konvaNode);
      });
      vue.onBeforeUnmount(() => {
        __konvaNode.destroy();
      });
      vue.watch(() => props.config, uploadKonva, { deep: true });
      expose({
        getStage,
        getNode
      });
      return () => {
        var _a;
        return vue.h("div", { ref: container }, (_a = slots.default) == null ? void 0 : _a.call(slots));
      };
    }
  };
  const EVENTS_NAMESPACE = ".vue-konva-event";
  const CONTAINERS = {
    Group: true,
    Layer: true,
    FastLayer: true,
    Label: true
  };
  function KonvaNode(nameNode) {
    return {
      props: {
        config: {
          type: Object,
          default: function() {
            return {};
          }
        },
        __useStrictMode: {
          type: Boolean
        }
      },
      setup(props, { attrs, slots, expose }) {
        const instance = vue.getCurrentInstance();
        const oldProps = vue.reactive({});
        const NodeClass = window.Konva[nameNode];
        if (!NodeClass) {
          console.error("vue-konva error: Can not find node " + nameNode);
          return;
        }
        const __konvaNode = new NodeClass();
        instance.__konvaNode = __konvaNode;
        instance.vnode.__konvaNode = __konvaNode;
        uploadKonva();
        function getNode() {
          return instance.__konvaNode;
        }
        function getStage() {
          return instance.__konvaNode;
        }
        function uploadKonva() {
          const events = {};
          for (var key in instance.vnode.props) {
            if (key.slice(0, 2) === "on") {
              events[key] = instance.vnode.props[key];
            }
          }
          const existingProps = oldProps || {};
          const newProps = {
            ...attrs,
            ...props.config,
            ...events
          };
          applyNodeProps(
            instance,
            newProps,
            existingProps,
            props.__useStrictMode
          );
          Object.assign(oldProps, newProps);
        }
        vue.onMounted(() => {
          const parentKonvaNode = findParentKonva(instance).__konvaNode;
          parentKonvaNode.add(__konvaNode);
          updatePicture(__konvaNode);
        });
        vue.onUnmounted(() => {
          updatePicture(__konvaNode);
          __konvaNode.destroy();
          __konvaNode.off(EVENTS_NAMESPACE);
        });
        vue.onUpdated(() => {
          uploadKonva();
          checkOrder(instance.subTree, __konvaNode);
        });
        vue.watch(() => props.config, uploadKonva, { deep: true });
        expose({
          getStage,
          getNode
        });
        const isContainer = CONTAINERS[nameNode];
        return isContainer ? () => {
          var _a;
          return vue.h("template", {}, (_a = slots.default) == null ? void 0 : _a.call(slots));
        } : () => null;
      }
    };
  }
  if (typeof window !== "undefined" && !window.Konva) {
    require("konva");
  }
  const KONVA_NODES = [
    "Layer",
    "FastLayer",
    "Group",
    "Label",
    "Rect",
    "Circle",
    "Ellipse",
    "Wedge",
    "Line",
    "Sprite",
    "Image",
    "Text",
    "TextPath",
    "Star",
    "Ring",
    "Arc",
    "Tag",
    "Path",
    "RegularPolygon",
    "Arrow",
    "Shape",
    "Transformer"
  ];
  const components = [
    {
      name: "Stage",
      component: Stage
    },
    ...KONVA_NODES.map((name) => ({
      name,
      component: KonvaNode(name)
    }))
  ];
  const VueKonva = {
    install: (app, options) => {
      let prefixToUse = componentPrefix;
      if (options && options.prefix) {
        prefixToUse = options.prefix;
      }
      components.forEach((k) => {
        app.component(`${prefixToUse}${k.name}`, k.component);
      });
    }
  };
  return VueKonva;
});
